@@ -2,6 +2,7 @@ from asyncio import Queue
 
 from interstate_py.connection_agent import ConnectionAgentBase
 from interstate_py.control.worker_task import AIOWorkerTask
+from interstate_py.error.error_assembly import ErrorAssembly
 from interstate_py.interstate_message import InterstateWireMessageType, InterstateWireMessage
 from interstate_py.log_factory import LogFactory
 from interstate_py.multiplex_message import MultiplexMessage
@@ -36,13 +37,12 @@ class StreamMultiplexer(AIOWorkerTask):
         # TODO: generify and decrease rx1 coupling
         if multiplex_message.type == InterstateWireMessageType.NEXT:
             self._log.info("[OUT][NEXT] - {}".format(multiplex_message))
-            await self._raw_outbound_queue.put(self._serialize_message(multiplex_message))
+            message = await self._serialize_message(multiplex_message)
+            await self._raw_outbound_queue.put(message)
         elif multiplex_message.type == InterstateWireMessageType.ERROR:
             self._log.info("[OUT][ERROR] - {}".format(multiplex_message))
-            await self._raw_outbound_queue.put(
-                InterstateWireMessage(multiplex_message.identity.encode(), InterstateWireMessageType.header("error"),
-                                      bytes(0))
-            )
+            self._log.warn("Error:", multiplex_message.payload)
+            await self._raw_outbound_queue.put(ErrorAssembly.to_on_error(multiplex_message))
             connection.close_outbound()
         elif multiplex_message.type == InterstateWireMessageType.COMPLETE:
             self._log.info("[OUT][COMPLETED] - {}".format(multiplex_message))
@@ -52,20 +52,29 @@ class StreamMultiplexer(AIOWorkerTask):
             )
             connection.close_outbound()
         elif multiplex_message.type == InterstateWireMessageType.PONG:
+            self._log.info("[OUT][PONG] - {}".format(multiplex_message))
             await self._raw_outbound_queue.put(
                 InterstateWireMessage(multiplex_message.identity.encode(), InterstateWireMessageType.header("pong"),
                                       bytes(0))
             )
-            connection.close_outbound()
         else:
             self._log.info("Can not handle message: {}".format(multiplex_message))
 
         self._outbound_queue.task_done()
 
-    def _serialize_message(self, multiplex_message: MultiplexMessage) -> InterstateWireMessage:
+    async def _serialize_message(self, multiplex_message: MultiplexMessage) -> InterstateWireMessage:
+        """
+        Serializes a messages using the given serializer. If something is going wrong during serialization an
+        on_error message is sent.
+
+        :param multiplex_message:
+        :return:
+        """
         try:
             serialized_payload = self._serialization.serialize(multiplex_message.payload)
             return InterstateWireMessage(multiplex_message.identity.encode(), InterstateWireMessageType.header("next"),
-                                         self._serialization.serialize(serialized_payload))
-        except:
+                                         serialized_payload)
+        except Exception as e:
             self._log.error("Something went wrong during serialization")
+            await self._raw_outbound_queue.put(ErrorAssembly.to_on_error(
+                MultiplexMessage(multiplex_message.identity, InterstateWireMessageType.ERROR, e)))
